#ifndef UI_ELEMENT_H
#define UI_ELEMENT_H

#include <string>
#include <bitset>

#include "window.h"
#include "UI_cursor.h"
#include "screenvert.h"
#include "screencolor.h"


//forward defs
class UI_command;
class Gui;

//class TiXmlNode;

//UI element flags (all elements have these flags)
const int DIRTY=1;
const int SELECTED=2;
const int ACTIVE=3;
const int VISIBLE=4;
const int INITIALIZED=5;
const int MOUSE_OVER=6;
const int DRAW_CORNER1_ROUND=7;
const int DRAW_CORNER2_ROUND=8;
const int DRAW_CORNER3_ROUND=9;
const int DRAW_CORNER4_ROUND=10;
const int HAS_TOOLTIP=11;
const int SHOW_TOOLTIP=12;

/************************************************************
Abstract base class for UI objects.
All UI objects are derived from this class. This allows for
lists of all derived types with runtime polymorphism.  Should
allow for cleaner code and a more intuitive codepath.
**********************************************************/
class UI_element
{
	public:
		UI_element(Gui* g);
		virtual ~UI_element();

		//main functions that all widgets must implement
		virtual void draw();
		virtual void process_event(NIGHT_Event event)=0;

		//mutators and selectors
		virtual void setParent(UI_element *new_parent) {myParent=new_parent;}
		virtual const UI_element* parent() const {return myParent;}

		virtual void setFlag(int flag, bool value) {myFlags.set(flag,value);}
		virtual void setFlag(std::string flag_name, bool value);
		virtual bool checkFlag(int flag) const {return myFlags.test(flag);}
		virtual void toggleFlag(int flag) {myFlags.flip(flag);}

		virtual void setCommand(UI_command* new_command) {myCommand=new_command;}
		virtual const UI_command* command() const {return myCommand;}

		//methods for pos/size
		virtual void setPosition(screenVert &p) {myPosition=p;}
		virtual void setPosition(int x, int y) {myPosition.x=x; myPosition.y=y;}
		virtual const screenVert* position() const {return &myPosition;}
		virtual void getPosition(int &x, int &y) {x=myPosition.x; y=myPosition.y;}

		virtual void setSize(screenVert &s) {mySize=s;}
		virtual void setSize(int x, int y) {mySize.x=x; mySize.y=y;}
		virtual const screenVert*  size() const {return &mySize;}
      virtual void getSize(int &x, int &y) {x=mySize.x; y=mySize.y;}

		virtual void setColor(screenColor &c) {myColor=c;}
		virtual void setColor(float r, float g, float b, float a);
		virtual const screenColor* color() const {return &myColor;}

		virtual void set_default_cursor(Cursor* new_cursor) {myDefaultCursor=new_cursor;}

		virtual int layer() const {return myLayer;}
		virtual void setLayer(int layer) {myLayer=layer;}

		virtual float scale() const {return myScale;}
		virtual void setScale(float scale) {myScale=scale;}

		virtual const Gui* gui() const {return myGui;}

	protected:
		virtual void drawBox() const;

      Gui* myGui;

		screenVert myPosition;
		screenVert mySize;

		std::bitset<32> myFlags;
		screenColor myColor;

		//parent of this UI element
		UI_element* myParent;
		UI_command* myCommand;

		Cursor* myDefaultCursor;

		int myLayer;
		float myScale;
};

class UI_element_creator
{
	public:
		UI_element_creator() {}
		virtual ~UI_element_creator() {}

		virtual UI_element* create();
};

#endif

