#include "UI_screen.h"

/************************************************************
Screen Constructor

Input: none
Outputs: None
Actions: Gives the UI_screen the name "Screen" and an ID of 0. 
Creates at least one area and adds it to it's lists, makes it
current.  Creates the edges all around that area, adds them 
to the UI_screen list, makes the last one current.
************************************************************/
UI_screen::UI_screen()
{
	pos.x=0; pos.y=0;
	size.x=SCREEN_WIDTH;
	size.y=SCREEN_HEIGHT;

	set_name("Screen");
	ID=0;
}

/************************************************************
Screen Copy Constructor

Input: Reference to a new screen
Outputs: a copy of the screen
Actions: Makes a deep copy of the current screen in the new 
reference.  Copies all the areas and edges (and they deep copy
themselves).
************************************************************/
UI_screen::UI_screen(const UI_screen &copy_screen)
{
	areas=copy_screen.areas;
	edges=copy_screen.edges;
}

/************************************************************
Screen Destructor

Input: none
Outputs: None
Actions: Deletes all the generated parts of this screen.
************************************************************/
UI_screen::~UI_screen()
{
	std::list<UI_edge*>::iterator edge_iter;
	std::list<UI_area*>::iterator area_iter;
	
	for(edge_iter=edges.begin();edge_iter!= edges.end(); edge_iter++)
	{
		if(edge_iter!=NULL)
		{
			delete *edge_iter;
			edge_iter=NULL;
		}
		edge_iter++;
	}
	
	for(area_iter=areas.begin();area_iter!= areas.end(); area_iter++)
	{
		if(area_iter!=NULL)
		{
			delete *area_iter;
			area_iter=NULL;
		}
	}
}

/************************************************************
Screen Draw

Input: Draw flags
Outputs: None
Actions: Draws all the elements of the screen using the draw
flags that are passed in.
************************************************************/
void UI_screen::draw()
{
	std::list<UI_edge*>::iterator edge_iter;
	std::list<UI_area*>::iterator area_iter;

	glPushName((GLuint)(this));

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();	
	glTranslatef(pos.x,pos.y,0.1);
	
	for(area_iter=areas.begin();area_iter!= areas.end(); area_iter++)
	{
		(*area_iter)->draw();
	}
	for(edge_iter=edges.begin();edge_iter!= edges.end(); edge_iter++)
	{
		(*edge_iter)->draw();
	}

	font_engine::set_font_color(1.0,1.0,0.0);
	font_engine::draw_string(300,400,HELVETICA10,"active edge: %x",active_edge);
	screen_vert mouse;

	//font_engine::draw_string(10,10,HELVETICA10,"active area: %x",active_area);

	NIGHT::get_mouse_coords(&mouse);
	font_engine::draw_string(300,420,HELVETICA10,"mouse: %i, %i",mouse.x, mouse.y);
	
	glPopMatrix();
	glPopName();	
}

/************************************************************
Screen Process Events

Input: An event
Outputs: None
Actions: Takes the current event and handles it, or passes it 
on to the current area or edge.
************************************************************/

void UI_screen::process_event(SDL_Event event)
{
	std::list<UI_edge*>::iterator edge_iter;
	std::list<UI_area*>::iterator area_iter;

	//process all sub elements	
	for(area_iter=areas.begin();area_iter!= areas.end(); area_iter++)
	{
		(*area_iter)->process_event(event);
		if((*area_iter)->check_flag(ACTIVE)==true)
		{
			active_area=(*area_iter);
		}
	}
	for(edge_iter=edges.begin();edge_iter!= edges.end(); edge_iter++)
	{
		(*edge_iter)->process_event(event);
		if((*edge_iter)->check_flag(ACTIVE)==true)
		{
			active_edge=(*edge_iter);
		}
	}
	
	//the Screen handles these events
	switch(event.type)
	{
	//change the current area/edge on mouse movements
		case SDL_MOUSEMOTION:
		{
			break;
		}
		default:
		{
			break;
		}
	}
	set_flag(MOUSE_OVER, false);
}
 
 
/************************************************************
Screen Get Edge
 
Input: none
Outputs: the edge the mouse is over or NULL
Actions: gets the edge that the mouse is over on this UI_screen,
NULL if there isn't one.
************************************************************/
UI_edge* UI_screen::get_edge()
{	
	std::list<UI_edge*>::iterator edge_iter;
	screen_vert mouse_point;

	screen_vert edge_pos, edge_size;

	//static function call to get the mouse coords
	NIGHT::get_mouse_coords(&mouse_point.x, &mouse_point.y);
	
	for(edge_iter=edges.begin(); edge_iter!=edges.end(); edge_iter++)
	{
		(*edge_iter)->get_pos(edge_pos);
		(*edge_iter)->get_size(edge_size);
		
		if ((*edge_iter)->is_horizontal())
		{
			//is it 2 pixels above or below the edge line and somewhere between the start and stop
			if (abs(mouse_point.y - edge_pos.y)<=2 && mouse_point.x >= edge_pos.x && mouse_point.x<=(edge_pos.x+edge_size.x))
			{
				return *edge_iter;
			}
		} 
		else 
		{
			//is it 2 pixes left or right of the edge line and somewhere between the top and bottom
			if (abs(mouse_point.x - edge_pos.x)<=2 && mouse_point.y >= edge_pos.y && mouse_point.y <=(edge_pos.y+edge_size.y))
			{
				return *edge_iter;
			}
		}
	}

	//didn't find one that matched.  Return NULL
	return NULL;
}

/************************************************************
Screen Join Areas

Input: The edge that called for joining the areas
Outputs: none
Actions: Joins the two areas that share the join_edge into one
edge.  The active area is the area that takes over.
************************************************************/
void UI_screen::join_areas()
{

}


/************************************************************
Screen Split Areas

Input: The area that is to be split
Outputs: none
Actions: Splits an area into two areas.  Uses the mouse coordinates
to determine the split point.  Will not split if the area is
not big enough.  The current area is resized to fit in the split
and the new area is an exact copy of the current area.  It is a
copy of the data, not a link to the same data (hence the need
for the copy constructor)
************************************************************/
void UI_screen::split_area()
{
	UI_area* new_area;
	UI_edge* new_edge;

	screen_vert split_point, area_pos, area_size;

	NIGHT::get_mouse_coords(&split_point.x, &split_point.y);	

	//is there really an area to split
	if(active_area==NULL)
	{
		logger::log("CRITICAL ERROR: Unable to split area:  Area does not exist");
		return;
	}
	else
	{
		active_area->get_pos(area_pos);
		active_area->get_size(area_size);
	}

	if(active_edge==NULL)
	{
		logger::log("CRITICAL ERROR: Unable to split area:  Cannot find calling edge");
		return;
	}

	//is the area big enough to split
	if((area_size.x <= 2*AREAMINX) or (area_size.y <= 2*AREAMINY))
	{
		logger::log("Unable to split area:  Area too small");
		return;
	}
	

	//create new area-deep copy of old area
	new_area=new UI_area(*active_area);

	//create new edge for between the areas
	new_edge=new UI_edge();

	//determine if it's a vertical or horizontal split
	if(active_edge->is_horizontal())
	{
		/* the current (or calling) edge is horizantal, so we want to create a verticle split
		   along the x axis just like in blender 
		*/
		//update edge
		new_edge->set_pos(split_point.x, area_pos.y);
		new_edge->set_size(split_point.x, area_size.y);

		//update areas
		active_area->set_size(split_point.x, area_pos.y);
		new_area->set_pos(split_point.x, area_pos.y);
	}
	else
	{
		/* the current (or calling) edge is vertical, so we want to create a horizantal split
		   along the y axis just like in blender 
		*/
		//update edges
		new_edge->set_pos(area_pos.x, split_point.y);
		new_edge->set_size(area_size.x, split_point.y);
		
		//update areas
		active_area->set_size(area_size.x, split_point.y);
		new_area->set_pos(area_pos.x, split_point.y);
	}

	//set the new edge's areas and parent
	new_edge->set_areas(active_area, new_area);
		
	//register the new area and edge with this UI_screen
	new_edge->set_parent(this);
	new_area->set_parent(this);
	areas.push_back(new_area);
	edges.push_back(new_edge);
}

void UI_screen::add_area()
{
	UI_area* temp_area;
	temp_area=new UI_area();

	//set the parent to itself to denote top element
	temp_area->set_parent(temp_area);

	areas.push_back(temp_area);
}

void UI_screen::add_area(UI_area* new_area)
{
	if(new_area!=NULL)
	{
		areas.push_back(new_area);
		new_area->set_parent(this);
	}
}

void UI_screen::add_edge()
{
	UI_edge* temp_edge;
	temp_edge=new UI_edge();

	//set the parent to itself to denote top element
	temp_edge->set_parent(temp_edge);

	edges.push_back(temp_edge);
}

void UI_screen::add_edge(UI_edge* new_edge)
{
	if(new_edge!=NULL)
	{
		edges.push_back(new_edge);
		new_edge->set_parent(this);
	}
}

void UI_screen::interactively_split()
{
}
